@@ -1,5 +1,5 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
  * Copyright (C) 2010
  * Martin Lambers <marlam@marlam.de>
@@ -56,7 +56,7 @@ private:
 
 public:
     audio_output_openal() throw ();
-    ~audio_output_openal();
+    virtual ~audio_output_openal();
 
     virtual void open(int channels, int rate, enum decoder::audio_sample_format format);
 
