@@ -1,5 +1,5 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
  * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
  *
@@ -33,6 +33,12 @@ extern "C"
 #include <cerrno>
 #include <cstring>
 
+#if HAVE_SYSCONF
+#  include <unistd.h>
+#else
+#  include <windows.h>
+#endif
+
 #include "debug.h"
 #include "blob.h"
 #include "exc.h"
@@ -42,7 +48,7 @@ extern "C"
 #include "decoder_ffmpeg.h"
 
 
-/* Hide the ffmpeg stuff so that their header files cannot cause problems
+/* Hide the FFmpeg stuff so that their header files cannot cause problems
  * in other source files. */
 struct internal_stuff
 {
@@ -79,6 +85,31 @@ static std::string my_av_strerror(int err)
     return std::string(b.ptr<const char>());
 }
 
+static int decoding_threads()
+{
+    // Use one decoding thread per processor.
+    static long n = -1;
+    if (n < 0)
+    {
+#ifdef HAVE_SYSCONF
+        n = sysconf(_SC_NPROCESSORS_ONLN);
+#else
+        SYSTEM_INFO si;
+        GetSystemInfo(&si);
+        n = si.dwNumberOfProcessors;
+#endif
+        if (n < 1)
+        {
+            n = 1;
+        }
+        else if (n > 64)
+        {
+            n = 64;
+        }
+    }
+    return n;
+}
+
 static void my_av_log(void *ptr, int level, const char *fmt, va_list vl)
 {
     static std::string line;
@@ -127,10 +158,10 @@ static void my_av_log(void *ptr, int level, const char *fmt, va_list vl)
         size_t n;
         while ((n = line.find('\n')) != std::string::npos)
         {
-            msg::msg(l, std::string("ffmpeg: ") + p + line.substr(0, n));
+            msg::msg(l, std::string("FFmpeg: ") + p + line.substr(0, n));
             line = line.substr(n + 1);
         }
-        msg::msg(l, std::string("ffmpeg: ") + p + line);
+        msg::msg(l, std::string("FFmpeg: ") + p + line);
         line.resize(0);
     }
 }
@@ -201,6 +232,11 @@ void decoder_ffmpeg::open(const std::string &filename)
             {
                 throw exc(filename + " stream " + str::from(i) + ": invalid frame size");
             }
+            _stuff->video_codec_ctxs[j]->thread_count = decoding_threads();
+            if (avcodec_thread_init(_stuff->video_codec_ctxs[j], _stuff->video_codec_ctxs[j]->thread_count) != 0)
+            {
+                _stuff->video_codec_ctxs[j]->thread_count = 1;
+            }
             _stuff->video_codecs.push_back(avcodec_find_decoder(_stuff->video_codec_ctxs[j]->codec_id));
             if (!_stuff->video_codecs[j])
             {
@@ -318,6 +354,7 @@ void decoder_ffmpeg::open(const std::string &filename)
                 static_cast<float>(video_frame_rate_numerator(i))
                 / static_cast<float>(video_frame_rate_denominator(i)),
                 video_duration(i) / 1e6f);
+        msg::inf("        using up to %d threads for decoding", _stuff->video_codec_ctxs.at(i)->thread_count);
     }
     for (int i = 0; i < audio_streams(); i++)
     {
@@ -767,13 +804,13 @@ void decoder_ffmpeg::close()
 std::vector<std::string> ffmpeg_versions()
 {
     std::vector<std::string> v;
-    v.push_back(str::asprintf("ffmpeg libavformat %d.%d.%d / %d.%d.%d",
+    v.push_back(str::asprintf("FFmpeg libavformat %d.%d.%d / %d.%d.%d",
                 LIBAVFORMAT_VERSION_MAJOR, LIBAVFORMAT_VERSION_MINOR, LIBAVFORMAT_VERSION_MICRO,
                 avformat_version() >> 16, avformat_version() >> 8 & 0xff, avformat_version() & 0xff));
-    v.push_back(str::asprintf("ffmpeg libavcodec %d.%d.%d / %d.%d.%d",
+    v.push_back(str::asprintf("FFmpeg libavcodec %d.%d.%d / %d.%d.%d",
                 LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO,
                 avcodec_version() >> 16, avcodec_version() >> 8 & 0xff, avcodec_version() & 0xff));
-    v.push_back(str::asprintf("ffmpeg libswscale %d.%d.%d / %d.%d.%d",
+    v.push_back(str::asprintf("FFmpeg libswscale %d.%d.%d / %d.%d.%d",
                 LIBSWSCALE_VERSION_MAJOR, LIBSWSCALE_VERSION_MINOR, LIBSWSCALE_VERSION_MICRO,
                 swscale_version() >> 16, swscale_version() >> 8 & 0xff, swscale_version() & 0xff));
     return v;
