@@ -1,7 +1,9 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
+ * Copyright (C) 2010
+ * Martin Lambers <marlam@marlam.de>
+ * Stefan Eilemann <eile@eyescale.ch>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,7 +59,7 @@ int main(int argc, char *argv[])
     log_levels.push_back("warning");
     log_levels.push_back("error");
     log_levels.push_back("quiet");
-    opt::val<std::string> log_level("log-level", 'l', opt::optional, log_levels, "info");
+    opt::val<std::string> log_level("log-level", 'l', opt::optional, log_levels, "");
     options.push_back(&log_level);
     std::vector<std::string> input_modes;
     input_modes.push_back("mono");
@@ -86,6 +88,7 @@ int main(int argc, char *argv[])
     video_output_modes.push_back("anaglyph-dubois");
     video_output_modes.push_back("stereo");
     video_output_modes.push_back("equalizer");
+    video_output_modes.push_back("equalizer-3d");
     opt::val<std::string> video_output_mode("output", 'o', opt::optional, video_output_modes, "");
     options.push_back(&video_output_mode);
     opt::flag fullscreen("fullscreen", 'f', opt::optional);
@@ -94,6 +97,8 @@ int main(int argc, char *argv[])
     options.push_back(&center);
     opt::flag swap_eyes("swap-eyes", 's', opt::optional);
     options.push_back(&swap_eyes);
+    opt::flag benchmark("benchmark", 'b', opt::optional);
+    options.push_back(&benchmark);
     // Accept some Equalizer options. These are passed to Equalizer for interpretation.
     opt::val<std::string> eq_server("eq-server", '\0', opt::optional);
     options.push_back(&eq_server);
@@ -114,7 +119,7 @@ int main(int argc, char *argv[])
     if (version.value())
     {
         msg::req("%s version %s", PACKAGE_NAME, VERSION);
-        msg::req("    Copyright (C) 2010 Martin Lambers <marlam@marlam.de>.");
+        msg::req("    Copyright (C) 2010 the Bino developers.");
         msg::req("    This is free software. You may redistribute copies of it");
         msg::req("    under the terms of the GNU General Public License.");
         msg::req("    There is NO WARRANTY, to the extent permitted by law.");
@@ -163,10 +168,12 @@ int main(int argc, char *argv[])
                 "    anaglyph-half-color  Red/cyan anaglyph, half color method\n"
                 "    anaglyph-dubois      Red/cyan anaglyph, Dubois method\n"
                 "    stereo               OpenGL quad-buffered stereo\n"
-                "    equalizer            Multi-display OpenGL using Equalizer\n"
+                "    equalizer            Multi-display OpenGL via Equalizer (2D setup)\n"
+                "    equalizer-3d         Multi-display OpenGL via Equalizer (3D setup)\n"
                 "  -f|--fullscreen      Fullscreen\n"
                 "  -c|--center          Center window on screen\n"
-                "  -s|--swap-eyes       Swap left/right view\n"        
+                "  -s|--swap-eyes       Swap left/right view\n"
+                "  -b|--benchmark       Benchmark mode (no audio, no timesync, show fps)\n"
                 "\n"
                 "Keyboard control:\n"
                 "  q or ESC             Quit\n"
@@ -193,7 +200,7 @@ int main(int argc, char *argv[])
     {
         try
         {
-            player_equalizer *player = new player_equalizer(&argc, argv);
+            player_equalizer *player = new player_equalizer(&argc, argv, true);
             delete player;
         }
         catch (std::exception &e)
@@ -206,9 +213,14 @@ int main(int argc, char *argv[])
 #endif
 
     bool equalizer = false;
+    bool equalizer_flat_screen = true;
     player_init_data init_data;
     init_data.log_level = msg::level();
-    if (log_level.value() == "debug")
+    if (log_level.value() == "")
+    {
+        init_data.log_level = msg::INF;
+    }
+    else if (log_level.value() == "debug")
     {
         init_data.log_level = msg::DBG;
     }
@@ -248,6 +260,16 @@ int main(int argc, char *argv[])
         equalizer = true;
         init_data.video_mode = video_output::mono_left;
     }
+    else if (video_output_mode.value() == "equalizer-3d")
+    {
+        equalizer = true;
+        equalizer_flat_screen = false;
+        init_data.video_mode = video_output::mono_left;
+    }
+    else if (video_output_mode.value() == "anaglyph")
+    {
+        init_data.video_mode = video_output::anaglyph_red_cyan_dubois;
+    }
     else if (video_output_mode.value() == "")
     {
         init_data.video_mode = video_output::automatic;
@@ -268,6 +290,11 @@ int main(int argc, char *argv[])
     {
         init_data.video_flags |= video_output::center;
     }
+    init_data.benchmark = benchmark.value();
+    if (init_data.benchmark)
+    {
+        msg::inf("benchmark mode: audio and time synchronization disabled");
+    }
 
     int retval = 0;
     player *player = NULL;
@@ -276,13 +303,19 @@ int main(int argc, char *argv[])
         if (equalizer)
         {
 #if HAVE_LIBEQUALIZER
-            player = new class player_equalizer(&argc, argv);
+            player = new class player_equalizer(&argc, argv, equalizer_flat_screen);
 #else
             throw exc("this version of Bino was compiled without support for Equalizer");
 #endif
         }
         else if (arguments.size() == 0 || show_gui.value())
         {
+            if (log_level.value() == "")
+            {
+                init_data.log_level = msg::WRN;         // Be silent by default when the GUI is used
+            }
+            init_data.video_state.fullscreen = false;   // GUI overrides fullscreen setting
+            init_data.video_flags = 0;                  // GUI overrides center flag
             player = new class player_qt();
         }
         else
