@@ -1,5 +1,5 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
  * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
  *
@@ -21,6 +21,7 @@
 
 #include <QCoreApplication>
 #include <QMainWindow>
+#include <QGridLayout>
 #include <QCloseEvent>
 #include <QMenu>
 #include <QMenuBar>
@@ -34,6 +35,8 @@
 #include <QComboBox>
 #include <QTimer>
 #include <QFile>
+#include <QByteArray>
+#include <QCryptographicHash>
 
 #include "player_qt.h"
 #include "qt_app.h"
@@ -41,8 +44,8 @@
 #include "lib_versions.h"
 
 
-player_qt_internal::player_qt_internal(video_output_opengl_qt *vo)
-    : player(player::master), _vo(vo), _playing(false)
+player_qt_internal::player_qt_internal(video_container_widget *container_widget)
+    : player(player::master), _container_widget(container_widget), _playing(false)
 {
 }
 
@@ -52,22 +55,18 @@ player_qt_internal::~player_qt_internal()
 
 void player_qt_internal::open(const player_init_data &init_data)
 {
+    // This is the same as player::open except for the creation
+    // of the video output. Here, we must pass the container widget.
     reset_playstate();
+    set_benchmark(init_data.benchmark);
     create_decoders(init_data.filenames);
     create_input(init_data.input_mode);
     create_audio_output();
-    set_video_output(_vo);
+    set_video_output(new video_output_opengl_qt(_container_widget));
     video_state() = init_data.video_state;
     open_video_output(init_data.video_mode, init_data.video_flags);
 }
 
-void player_qt_internal::close()
-{
-    player::set_video_output(NULL);
-    player::close();
-    player::set_video_output(_vo);
-}
-
 void player_qt_internal::receive_cmd(const command &cmd)
 {
     if (cmd.type == command::toggle_play && !_playing)
@@ -123,10 +122,13 @@ void player_qt_internal::force_stop()
     notify(notification::play, false, false);
 }
 
-QWidget *player_qt_internal::video_output_widget()
+void player_qt_internal::move_event()
 {
-    video_output_opengl_qt *vo = static_cast<video_output_opengl_qt *>(player::get_video_output());
-    return vo->widget();
+    video_output_opengl_qt *vo = static_cast<video_output_opengl_qt *>(get_video_output());
+    if (vo)
+    {
+        vo->move_event();
+    }
 }
 
 
@@ -570,18 +572,17 @@ main_window::main_window(QSettings *settings, const player_init_data &init_data)
 
     // Central widget
     QWidget *central_widget = new QWidget(this);
-    _layout = new QGridLayout();
-    _video_output = new video_output_opengl_qt(central_widget);
-    _video_output->open(decoder::frame_format_bgra32, 1, 1, 1.0f, video_output::mono_left, video_output_state(), 0, 0, 0);
-    _video_widget = _video_output->widget();
-    _layout->addWidget(_video_widget, 0, 0);
+    QGridLayout *layout = new QGridLayout();
+    _video_container_widget = new video_container_widget(central_widget);
+    connect(_video_container_widget, SIGNAL(move_event()), this, SLOT(move_event()));
+    layout->addWidget(_video_container_widget, 0, 0);
     _in_out_widget = new in_out_widget(_settings, central_widget);
-    _layout->addWidget(_in_out_widget, 1, 0);
+    layout->addWidget(_in_out_widget, 1, 0);
     _controls_widget = new controls_widget(_settings, central_widget);
-    _layout->addWidget(_controls_widget, 2, 0);
-    _layout->setRowStretch(0, 1);
-    _layout->setColumnStretch(0, 1);
-    central_widget->setLayout(_layout);
+    layout->addWidget(_controls_widget, 2, 0);
+    layout->setRowStretch(0, 1);
+    layout->setColumnStretch(0, 1);
+    central_widget->setLayout(layout);
     setCentralWidget(central_widget);
 
     // Menus
@@ -614,7 +615,7 @@ main_window::main_window(QSettings *settings, const player_init_data &init_data)
     show();     // Must happen before opening initial files!
 
     // Player and timer
-    _player = new player_qt_internal(_video_output);
+    _player = new player_qt_internal(_video_container_widget);
     _timer = new QTimer(this);
     connect(_timer, SIGNAL(timeout()), this, SLOT(playloop_step()));
 
@@ -628,6 +629,10 @@ main_window::main_window(QSettings *settings, const player_init_data &init_data)
         }
         open(filenames, false);
     }
+    else
+    {
+        adjustSize();   // Adjust size for initial dummy video widget
+    }
 }
 
 main_window::~main_window()
@@ -650,10 +655,6 @@ bool main_window::open_player()
         QMessageBox::critical(this, "Error", tr("%1").arg(e.what()));
         return false;
     }
-    _layout->removeWidget(_video_widget);
-    delete _video_widget;
-    _video_widget = _video_output->widget();
-    _layout->addWidget(_video_widget, 0, 0);
     adjustSize();
     return true;
 }
@@ -664,6 +665,9 @@ void main_window::receive_notification(const notification &note)
     {
         if (note.current.flag)
         {
+            // Close and re-open the player. This resets the video state in case
+            // we played it before, and it sets the input/output modes to the
+            // current choice.
             _player->close();
             _init_data.input_mode = _in_out_widget->input_mode();
             _init_data.video_mode = _in_out_widget->video_mode();
@@ -671,13 +675,17 @@ void main_window::receive_notification(const notification &note)
             {
                 _stop_request = true;
             }
+            // Remember the input mode of this video, using an SHA1 hash of its
+            // filename.
             _settings->beginGroup("Video");
             if (_init_data.filenames.size() == 1)
             {
-                _settings->setValue(QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName(),
-                        QString(input::mode_name(_init_data.input_mode).c_str()));
+                QString name = QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName();
+                QByteArray hash = QCryptographicHash::hash(name.toUtf8(), QCryptographicHash::Sha1);
+                _settings->setValue(QString(hash.toHex()), QString(input::mode_name(_init_data.input_mode).c_str()));
             }
             _settings->endGroup();
+            // Remember the 2D or 3D video output mode.
             _settings->beginGroup("Session");
             if (_init_data.input_mode == input::mono)
             {
@@ -688,9 +696,12 @@ void main_window::receive_notification(const notification &note)
                 _settings->setValue("3d-output-mode", QString(video_output::mode_name(_init_data.video_mode).c_str()));
             }
             _settings->endGroup();
+            // Update widgets: we're now playing
             _in_out_widget->update(_init_data, true, true);
             _controls_widget->update(_init_data, true, true);
-            _video_widget->setFocus(Qt::OtherFocusReason);
+            // Give the keyboard focus to the video widget
+            _video_container_widget->setFocus(Qt::OtherFocusReason);
+            // Start the play loop
             _timer->start(0);
         }
         else
@@ -701,6 +712,24 @@ void main_window::receive_notification(const notification &note)
     }
 }
 
+void main_window::moveEvent(QMoveEvent *)
+{
+    move_event();
+}
+
+void main_window::closeEvent(QCloseEvent *event)
+{
+    event->accept();
+}
+
+void main_window::move_event()
+{
+    if (_player)
+    {
+        _player->move_event();
+    }
+}
+
 void main_window::playloop_step()
 {
     if (_stop_request)
@@ -715,24 +744,6 @@ void main_window::playloop_step()
     }
 }
 
-void main_window::moveEvent(QMoveEvent *)
-{
-    if (_video_output && _video_widget)
-    {
-        if (_video_output->mode() == video_output::even_odd_rows
-                || _video_output->mode() == video_output::even_odd_columns
-                || _video_output->mode() == video_output::checkerboard)
-        {
-            _video_widget->update();
-        }
-    }
-}
-
-void main_window::closeEvent(QCloseEvent *event)
-{
-    event->accept();
-}
-
 void main_window::open(QStringList filenames, bool automatic)
 {
     _player->force_stop();
@@ -752,9 +763,10 @@ void main_window::open(QStringList filenames, bool automatic)
         _settings->beginGroup("Video");
         if (_init_data.filenames.size() == 1)
         {
+            QString name = QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName();
+            QByteArray hash = QCryptographicHash::hash(name.toUtf8(), QCryptographicHash::Sha1);
             QString fallback_mode_name = QString(input::mode_name(_player->input_mode()).c_str());
-            QString filename = QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName();
-            QString mode_name = _settings->value(filename, fallback_mode_name).toString();
+            QString mode_name = _settings->value(QString(hash.toHex()), fallback_mode_name).toString();
             _init_data.input_mode = input::mode_from_name(mode_name.toStdString());
         }
         else
@@ -877,7 +889,7 @@ void main_window::help_about()
 {
     QString blurb = tr(
             "<p>%1 version %2</p>"
-            "<p>Copyright (C) 2010 Martin Lambers and others.<br>"
+            "<p>Copyright (C) 2010 the Bino developers.<br>"
             "This is free software. You may redistribute copies of it<br>"
             "under the terms of the <a href=\"http://www.gnu.org/licenses/gpl.html\">"
             "GNU General Public License</a>.<br>"
