@@ -1,5 +1,5 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
  * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
  *
@@ -24,7 +24,6 @@
 #include <QWidget>
 #include <QComboBox>
 #include <QPushButton>
-#include <QGridLayout>
 #include <QTimer>
 #include <QSettings>
 
@@ -37,23 +36,21 @@
 class player_qt_internal : public player, public controller
 {
 private:
-    video_output_opengl_qt *_vo;
+    video_container_widget *_container_widget;
     bool _playing;
 
 public:
-    player_qt_internal(video_output_opengl_qt *vo);
-    ~player_qt_internal();
+    player_qt_internal(video_container_widget *container_widget);
+    virtual ~player_qt_internal();
 
     virtual void open(const player_init_data &init_data);
-    virtual void close();
     virtual void receive_cmd(const command &cmd);
 
     virtual void receive_notification(const notification &note);
 
     bool playloop_step();
     void force_stop();
-
-    QWidget *video_output_widget();
+    void move_event();
 };
 
 class in_out_widget : public QWidget, public controller
@@ -80,7 +77,7 @@ private slots:
 
 public:
     in_out_widget(QSettings *settings, QWidget *parent);
-    ~in_out_widget();
+    virtual ~in_out_widget();
 
     void update(const player_init_data &init_data, bool have_valid_input, bool playing);
 
@@ -120,7 +117,7 @@ private slots:
 
 public:
     controls_widget(QSettings *settings, QWidget *parent);
-    ~controls_widget();
+    virtual ~controls_widget();
 
     void update(const player_init_data &init_data, bool have_valid_input, bool playing);
     virtual void receive_notification(const notification &note);
@@ -132,12 +129,10 @@ class main_window : public QMainWindow, public controller
 
 private:
     QSettings *_settings;
-    player_qt_internal *_player;
-    video_output_opengl_qt *_video_output;
-    video_output_opengl_qt_widget *_video_widget;
+    video_container_widget *_video_container_widget;
     in_out_widget *_in_out_widget;
     controls_widget *_controls_widget;
-    QGridLayout *_layout;
+    player_qt_internal *_player;
     QTimer *_timer;
     player_init_data _init_data;
     bool _stop_request;
@@ -146,6 +141,7 @@ private:
     void open(QStringList filenames, bool automatic);
 
 private slots:
+    void move_event();
     void playloop_step();
     void file_open();
     void file_open_url();
@@ -158,7 +154,7 @@ protected:
 
 public:
     main_window(QSettings *settings, const player_init_data &init_data);
-    ~main_window();
+    virtual ~main_window();
 
     virtual void receive_notification(const notification &note);
 };
@@ -172,7 +168,7 @@ private:
 
 public:
     player_qt();
-    ~player_qt();
+    virtual ~player_qt();
 
     QSettings *settings()
     {
