@@ -1,7 +1,9 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
+ * Copyright (C) 2010
+ * Martin Lambers <marlam@marlam.de>
+ * Frédéric Devernay <frederic.devernay@inrialpes.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,16 +83,16 @@ std::string video_output::mode_name(enum mode m)
         name = "checkerboard";
         break;
     case anaglyph_red_cyan_monochrome:
-        name = "anaglyph-red-cyan-monochrome";
+        name = "anaglyph-monochrome";
         break;
     case anaglyph_red_cyan_full_color:
-        name = "anaglyph-red-cyan-full-color";
+        name = "anaglyph-full-color";
         break;
     case anaglyph_red_cyan_half_color:
-        name = "anaglyph-red-cyan-half-color";
+        name = "anaglyph-half-color";
         break;
     case anaglyph_red_cyan_dubois:
-        name = "anaglyph-red-cyan-dubois";
+        name = "anaglyph-dubois";
         break;
     }
     return name;
@@ -147,19 +149,19 @@ enum video_output::mode video_output::mode_from_name(const std::string &name, bo
     {
         m = checkerboard;
     }
-    else if (name == "anaglyph-red-cyan-monochrome")
+    else if (name == "anaglyph-monochrome")
     {
         m = anaglyph_red_cyan_monochrome;
     }
-    else if (name == "anaglyph-red-cyan-full-color")
+    else if (name == "anaglyph-full-color")
     {
         m = anaglyph_red_cyan_full_color;
     }
-    else if (name == "anaglyph-red-cyan-half-color")
+    else if (name == "anaglyph-half-color")
     {
         m = anaglyph_red_cyan_half_color;
     }
-    else if (name == "anaglyph-red-cyan-dubois")
+    else if (name == "anaglyph" || name == "anaglyph-dubois")
     {
         m = anaglyph_red_cyan_dubois;
     }
