@@ -1,5 +1,5 @@
 /*
- * This file is part of bino, a program to play stereoscopic videos.
+ * This file is part of bino, a 3D video player.
  *
  * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
  *
@@ -82,7 +82,7 @@ public:
 public:
     /* Constructor, Destructor */
     video_output(bool receive_notifications = true) throw ();
-    ~video_output();
+    virtual ~video_output();
 
     /* Get capabilities */
     virtual bool supports_stereo() = 0; // Support for OpenGL quad buffered stereo?
